@@ -3,9 +3,5 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from . import (
-    backfill_make_list_elem_type,
-    expand_tf_lstm,
-    tf_lstm_to_core_lstm,
-    tf_passes
-)
+from . import (backfill_make_list_elem_type, expand_tf_lstm,
+               tf_lstm_to_core_lstm, tf_passes)
