@@ -3,9 +3,9 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
-import logging
+from coremltools import _logger as logger
 from coremltools.converters._profile_utils import _profile
+from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
 
 
 @_profile
@@ -25,8 +25,8 @@ def tensorflow_passes(prog):
 
     prog.validate()
     for p in passes:
-        logging.info('Performing passes for tf1 frontend: "{}"'.format(p))
+        logger.info('Performing passes for tf1 frontend: "{}"'.format(p))
         PASS_REGISTRY[p](prog)
         prog.validate()
 
-    logging.debug("Program after tf1 frontend passes:\n{}".format(prog))
+    logger.debug("Program after tf1 frontend passes:\n{}".format(prog))
