@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
+# Copyright (c) 2021, Apple Inc. All rights reserved.
+#
+# Use of this source code is governed by a BSD-3-clause license that can be
+# found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 import numpy as np
 
-from coremltools.converters.mil.mil import types
-from ..parsed_tf_node import ParsedTFNode
-from ..basic_graph_ops import replace_source, delete_node
+from ..basic_graph_ops import delete_node, replace_source
 
 
 def _try_same(input_h, input_w, W_h, W_w, dilation_factor, padding, crop):
