@@ -4,32 +4,27 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from collections import OrderedDict
-import logging as _logging
+
 import numpy as _np
 import torch as _torch
 
+from coremltools import _logger as logger
 from coremltools._deps import version_lt
-from coremltools.converters.mil._deployment_compatibility import AvailableTarget as _target
+from coremltools.converters.mil._deployment_compatibility import \
+    AvailableTarget as _target
 from coremltools.converters.mil.input_types import ImageType
-from coremltools.converters.mil.mil import (
-    Builder as mb,
-    Function,
-    types,
-    Program
-)
+from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil import Function, Program, types
 
+from .._utils import get_output_names
 from .internal_graph import InternalTorchIRGraph
 from .ops import convert_nodes
-from .torch_op_registry import _TORCH_OPS_REGISTRY
-from .torchir_passes import (
-    flatten_graph_input_values,
-    flatten_graph_output_values,
-    generate_tensor_assignment_ops,
-    transform_inplace_ops,
-    remove_getattr_nodes
-)
 from .ssa_passes.torch_passes import torch_passes
-from .._utils import get_output_names
+from .torch_op_registry import _TORCH_OPS_REGISTRY
+from .torchir_passes import (flatten_graph_input_values,
+                             flatten_graph_output_values,
+                             generate_tensor_assignment_ops,
+                             remove_getattr_nodes, transform_inplace_ops)
 
 torch_to_mil_types = {
     _torch.bool: types.bool,
@@ -40,6 +35,7 @@ torch_to_mil_types = {
     _torch.int64: types.int32,
 }
 
+
 mil_to_torch_types = {v: k for k, v in torch_to_mil_types.items()}
 
 
@@ -146,24 +142,28 @@ class TorchConverter:
             cut_at_symbols: A list of internal symbol name strings. Graph conversion will
                 terminate once these symbols have been generated. For debugging use
                 only. See kwarg in load.py.
-            opset_version: An int represents the coreml opset version
+            opset_version: An int represents the Core ML opset version
         """
-
         assert isinstance(torchscript, _torch.jit.ScriptModule)
+
         self.inputs = inputs
         for idx, inp in enumerate(self.inputs):
             if isinstance(inp, ImageType) and self.inputs[idx].channel_first is None:
                 self.inputs[idx].channel_first = True
+
         self.torchscript = torchscript
         self.outputs = outputs
         self.output_names = get_output_names(self.outputs)
         self.opset_version = _target(opset_version) if opset_version is not None else None
         self.context = TranscriptionContext()
+
         raw_graph, params_dict = self._expand_and_optimize_ir(self.torchscript)
         self.params_dict = params_dict
         self.graph = InternalTorchIRGraph(
             raw_graph, params_dict, self.inputs, cut_at_symbols
         )
+
+        # Apply Torch IR passes
         passes = [
             transform_inplace_ops,
             flatten_graph_input_values,
@@ -173,7 +173,8 @@ class TorchConverter:
         ]
         for p in passes:
             p(self.graph)
-        self.inputs = [v for v in self.graph.inputs.values()]
+
+        self.inputs = list(self.graph.inputs.values())
         self.torch_passes = torch_passes
         self._prog = Program()
 
@@ -224,7 +225,7 @@ class TorchConverter:
             self.context.add(const)
 
     def convert(self):
-        _logging.info("Converting graph.")
+        logger.info("Converting graph.")
 
         # This will hold the converted model.
         prog = self._prog
@@ -275,7 +276,7 @@ class TorchConverter:
             for g in graph_outputs:
                 if g is None:
                     msg = "Droping output {} which is None"
-                    _logging.warning(msg.format(g))
+                    logger.warning(msg.format(g))
             graph_outputs = [g for g in graph_outputs if g is not None]
 
             # Output renaming occurs
@@ -332,7 +333,7 @@ class TorchConverter:
 
         And a dictionary {"linear.weight": ..., "linear.bias": ...} is returned, to record the parameters values.
         Note that, those GetAttr nodes are still in the torch ir graph, but they would be removed in a latter
-        graph pass in the coreml torch internal graph
+        graph pass in the coremltools torch internal graph
 
         """
 
@@ -369,7 +370,8 @@ class TorchConverter:
         def _check_is_tensor(node, module):
             if not isinstance(module, _torch.Tensor):
                 return False
-            assert str(node.output().type()) == "Tensor"
+            if str(node.output().type()) not in ("Tensor", "Optional[Tensor]"):
+                raise TypeError("Type \"{}\" not supported".format(node.output().type()))
             return True
 
         def _check_is_quantized_tensor(node, module):
