@@ -3,8 +3,7 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import logging
-
+from coremltools import _logger as logger
 from coremltools.converters._profile_utils import _profile
 from coremltools.converters.mil.mil.passes.pass_registry import PASS_REGISTRY
 
@@ -21,8 +20,8 @@ def torch_passes(prog):
 
     prog.validate()
     for p in passes:
-        logging.info('Performing passes for torch frontend: "{}"'.format(p))
+        logger.info('Performing passes for torch frontend: "{}"'.format(p))
         PASS_REGISTRY[p](prog)
         prog.validate()
 
-    logging.debug("Program after torch frontend passes:\n{}".format(prog))
+    logger.debug("Program after torch frontend passes:\n{}".format(prog))
