@@ -4,20 +4,16 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from collections import OrderedDict
+
 import numpy as np
 import pytest
 import torch
 
-from ..internal_graph import (
-    InternalTorchIRBlock,
-    InternalTorchIRGraph,
-    InternalTorchIRNode
-)
-from ..torchir_passes import (
-    flatten_graph_input_values,
-    flatten_graph_output_values,
-    transform_inplace_ops
-)
+from ..internal_graph import (InternalTorchIRBlock, InternalTorchIRGraph,
+                              InternalTorchIRNode)
+from ..torchir_passes import (flatten_graph_input_values,
+                              flatten_graph_output_values,
+                              transform_inplace_ops)
 
 
 def _build_flattening_test_graph():
@@ -171,7 +167,6 @@ class TestTorchPasses:
         np.testing.assert_equal(len(graph.outputs), 1)
         np.testing.assert_equal(graph.outputs[0], graph.nodes[-1].outputs[0])
 
-
     def test_transform_inplace_ops_loop(self):
         # The test graph is:
         #    graph(
@@ -265,7 +260,6 @@ class TestTorchPasses:
         # That graph output should now be the output of the graph.
         np.testing.assert_equal(loop_node.outputs[0], graph.outputs[0])
 
-
     @pytest.mark.xfail(reason="rdar://64235006")
     def test_transform_inplace_ops_if(self):
         # The test graph is:
