@@ -8,13 +8,16 @@ import pytest
 import torch
 import torch.nn as nn
 
-from ..converter import torch_to_mil_types
-from coremltools import TensorType, RangeDim
-from coremltools._deps import _IS_MACOS
+import coremltools as ct
 import coremltools.models.utils as coremltoolsutils
-from coremltools.converters.mil.mil.types.type_mapping import nptype_from_builtin
+from coremltools import RangeDim, TensorType
+from coremltools._deps import _IS_MACOS
+from coremltools.converters.mil.mil.types.type_mapping import \
+    nptype_from_builtin
 from coremltools.converters.mil.testing_utils import ct_convert
 
+from ..converter import torch_to_mil_types
+
 
 class ModuleWrapper(nn.Module):
     """
@@ -33,9 +36,9 @@ class ModuleWrapper(nn.Module):
 np.random.seed(1984)
 
 
-def _flatten(object):
+def _flatten(objects):
     flattened_list = []
-    for item in object:
+    for item in objects:
         if isinstance(item, (list, tuple)):
             flattened_list.extend(_flatten(item))
         else:
@@ -71,7 +74,8 @@ def convert_to_coreml_inputs(input_description, inputs):
 
 
 def convert_to_mlmodel(model_spec, tensor_inputs, backend=("neuralnetwork", "fp32"),
-                       converter_input_type=None,use_cpu_for_conversion=False):
+                       converter_input_type=None, compute_unit=ct.ComputeUnit.CPU_ONLY,
+                       minimum_deployment_target=None):
     def _convert_to_inputtype(inputs):
         if isinstance(inputs, list):
             return [_convert_to_inputtype(x) for x in inputs]
@@ -90,8 +94,10 @@ def convert_to_mlmodel(model_spec, tensor_inputs, backend=("neuralnetwork", "fp3
         inputs = list(_convert_to_inputtype(tensor_inputs))
     else:
         inputs = converter_input_type
+        
     return ct_convert(model_spec, inputs=inputs, convert_to=backend,
-                      source="pytorch", useCPUOnly=use_cpu_for_conversion)
+                      source="pytorch", compute_units=compute_unit,
+                      minimum_deployment_target=minimum_deployment_target)
 
 
 def generate_input_data(input_size, rand_range=(0, 1)):
@@ -133,11 +139,17 @@ def flatten_and_detach_torch_results(torch_results):
     return [torch_results.detach().numpy()]
 
 
-def convert_and_compare(input_data, model_spec,
-                        expected_results=None, atol=1e-4,
-                        backend=("neuralnetwork", "fp32"),
-                        converter_input_type=None,
-                        use_cpu_for_conversion=False):
+def convert_and_compare(
+    input_data,
+    model_spec,
+    expected_results=None,
+    atol=1e-4,
+    rtol=1e-05,
+    backend=("neuralnetwork", "fp32"),
+    converter_input_type=None,
+    compute_unit=ct.ComputeUnit.CPU_ONLY,
+    minimum_deployment_target=None
+):
     """
     If expected results is not set, it will by default
     be set to the flattened output of the torch model.
@@ -145,10 +157,6 @@ def convert_and_compare(input_data, model_spec,
     Inputs:
 
     - input_data: torch.tensor or list[torch.tensor]
-    - use_cpu_for_conversion: bool
-        Argument which is passed as is to the unified converter API.
-        That is, "ct.convert(...., useCPUOnly=use_cpu_for_conversion)"
-        It forces the model to be loaded on the CPU context, post conversion.
     """
     if isinstance(model_spec, str):
         torch_model = torch.jit.load(model_spec)
@@ -164,7 +172,8 @@ def convert_and_compare(input_data, model_spec,
     expected_results = flatten_and_detach_torch_results(expected_results)
     mlmodel = convert_to_mlmodel(model_spec, input_data, backend=backend,
                                  converter_input_type=converter_input_type,
-                                 use_cpu_for_conversion=use_cpu_for_conversion)
+                                 compute_unit=compute_unit,
+                                 minimum_deployment_target=minimum_deployment_target,)
 
     coreml_inputs = convert_to_coreml_inputs(mlmodel.input_description, input_data)
 
@@ -172,11 +181,12 @@ def convert_and_compare(input_data, model_spec,
         return model_spec, mlmodel, coreml_inputs, None
 
     _, dtype = backend
-    if dtype == "fp16":
+    if mlmodel.compute_unit != ct.ComputeUnit.CPU_ONLY or (dtype == "fp16"):
         atol = max(atol * 100.0, 5e-1)
+        rtol = max(rtol * 100.0, 5e-2)
 
-    if not coremltoolsutils._has_custom_layer(mlmodel.get_spec()):
-        coreml_results = mlmodel.predict(coreml_inputs, useCPUOnly=True)
+    if not coremltoolsutils._has_custom_layer(mlmodel._spec):
+        coreml_results = mlmodel.predict(coreml_inputs)
         sorted_coreml_results = [
             coreml_results[key] for key in sorted(coreml_results.keys())
         ]
@@ -185,12 +195,11 @@ def convert_and_compare(input_data, model_spec,
             if torch_result.shape == ():
                 torch_result = np.array([torch_result])
             np.testing.assert_equal(coreml_result.shape, torch_result.shape)
-            np.testing.assert_allclose(coreml_result, torch_result,
-                                       atol=atol)
+            np.testing.assert_allclose(coreml_result, torch_result, atol=atol, rtol=rtol)
     return model_spec, mlmodel, coreml_inputs, coreml_results
 
 
-class TorchBaseTest(object):
+class TorchBaseTest:
     testclassname = ''
     testmodelname = ''
 
@@ -199,16 +208,20 @@ class TorchBaseTest(object):
         TorchBaseTest.testclassname = type(self).__name__
         TorchBaseTest.testmodelname = request.node.name
 
-    def teardown_method(self, method):
-        pass
-
     @staticmethod
     def run_compare_torch(
-            input_data, model, expected_results=None, places=5,
-            input_as_shape=True, backend=("neuralnetwork", "fp32"),
-            rand_range=(0.0, 1.0), use_scripting=False,
+            input_data,
+            model,
+            expected_results=None,
+            atol=1e-04,
+            rtol=1e-05,
+            input_as_shape=True,
+            backend=("neuralnetwork", "fp32"),
+            rand_range=(-1.0, 1.0),
+            use_scripting=False,
             converter_input_type=None,
-            use_cpu_for_conversion=False,
+            compute_unit=ct.ComputeUnit.CPU_ONLY,
+            minimum_deployment_target=None,
     ):
         """
         Traces a model and runs a numerical test.
@@ -229,10 +242,15 @@ class TorchBaseTest(object):
 
         model_spec, mlmodel, coreml_inputs, coreml_results = \
             convert_and_compare(
-                input_data, model_spec, expected_results=expected_results,
-                atol=10.0 ** -places, backend=backend,
+                input_data,
+                model_spec,
+                expected_results=expected_results,
+                atol=atol,
+                rtol=rtol,
+                backend=backend,
                 converter_input_type=converter_input_type,
-                use_cpu_for_conversion=use_cpu_for_conversion,
+                compute_unit=compute_unit,
+                minimum_deployment_target=minimum_deployment_target,
             )
 
         return model_spec, mlmodel, coreml_inputs, coreml_results, \
