@@ -4,14 +4,15 @@
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
 from enum import Enum
+
 import numpy as np
 
-from coremltools.converters.mil.mil.types.symbolic import is_symbolic
 from coremltools.converters.mil.mil import types
+from coremltools.converters.mil.mil.types.symbolic import is_symbolic
 from coremltools.converters.mil.mil.types.type_mapping import (
-    numpy_type_to_builtin_type,
-    is_builtin,
-)
+    is_builtin, numpy_type_to_builtin_type)
+
+
 
 class ColorLayout(Enum):
     RGB = "RGB"
@@ -19,6 +20,7 @@ class ColorLayout(Enum):
     GRAYSCALE = "G"
     GRAYSCALE_FLOAT16 = "G_FLOAT16"
 
+
 class ClassifierConfig:
     def __init__(
         self,
@@ -93,10 +95,10 @@ class ImageType(InputType):
 
         Parameters
         ----------
-        scale: (float)
+        scale: float or list of floats
             The scaling factor for all values in the image channels.
 
-        bias: float or list of float
+        bias: float or list of floats
             * If ``color_layout`` is ``ct.colorlayout.GRAYSCALE`` or
               ``ct.colorlayout.GRAYSCALE_FLOAT16``, bias would be a ``float``.
             * If ``color_layout`` is ``ct.colorlayout.RGB`` or ``ct.colorlayout.BGR``,
