@@ -3,20 +3,18 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from collections import Counter, OrderedDict
 import copy
-import logging
+from collections import Counter, OrderedDict
+
+from coremltools import _OPSET, _logger as logger
+from coremltools.converters.mil._deployment_compatibility import \
+    AvailableTarget as _target
 
 from . import SPACES, types
-from .types.symbolic import (
-    k_used_symbols,
-    k_num_internal_syms,
-    is_symbolic,
-)
-from .var import Var, InternalVar
+from .types.symbolic import is_symbolic, k_used_symbols
+from .var import InternalVar, Var
 from .visitors.dot_visitor import DotVisitor
 
-
 # BLOCK_STACK[-1] is the current block
 BLOCK_STACK = []
 DEBUG = False
@@ -26,12 +24,23 @@ def curr_block():
         raise ValueError("Must call Builder inside an Function" + " or Block")
     return BLOCK_STACK[-1]
 
+def curr_opset_version():
+    block = curr_block()
+    while not isinstance(block, Function):
+        block = block.outer_op.enclosing_block
+    return block.opset_version
+
+def is_current_opset_version_compatible_with(opset_version):
+    if curr_opset_version() is None:
+        return opset_version <= _target.iOS13
+    return curr_opset_version() >= opset_version
+
 
 class InvalidBlockStateError(Exception):
     pass
 
 
-class Block(object):
+class Block:
     __slots__ = [
         "name",
         "_block_inputs",
@@ -225,6 +234,49 @@ class Block(object):
     def outputs(self):
         return self._outputs
 
+    def is_var_visible_in_block(self, var, upto_op_with_id=None):
+        """
+        Checks if a var is visible to ops starting from id=`upto_op_with_id` inside the block.
+
+        Var is visible if
+        - It is the output of a const op, or
+        - It is the output of "preceding" operations in that block, or
+        - It is visible in the enclosing block, or
+        - It is either a block or a function input
+
+        If upto_op_with_id is None, outputs of all operations inside the block are visible to
+        that block.
+        """
+
+        if var in self._internal_vars:
+            return True
+
+        inputs = self.function_inputs if isinstance(self, Function) else self.inputs
+        if var in inputs:
+            return True
+
+        idx = len(self.operations) if upto_op_with_id is None else upto_op_with_id
+
+        for i in range(idx-1, -1, -1):
+            op_outputs = self.operations[i].outputs
+            if op_outputs is not None and var in op_outputs:
+                return True
+
+        if self.outer_op is not None:
+            enclosing_block = self.outer_op.enclosing_block
+            outer_op_id = enclosing_block.find_op_id_in_block(self.outer_op)
+            if enclosing_block.is_var_visible_in_block(var, upto_op_with_id=outer_op_id):
+                return True
+
+        return False
+
+    def find_op_id_in_block(self, target_op):
+        try:
+            idx = self.operations.index(target_op)
+        except ValueError:
+            raise ValueError("Op {} not found in {}: {}".format(target_op.name, self.name, self))
+        return idx
+
     def set_outputs(self, outputs):
         """
         outputs: list[Var]
@@ -233,25 +285,24 @@ class Block(object):
             raise ValueError("Outputs must be list of Vars")
 
         self.validate()
-        visible_vars = self._visible_vars_from_enclosing_block()
-        _, visible_vars_in_block = self._visible_vars_in_block()
-        visible_vars.update(visible_vars_in_block)
         for ov in outputs:
-            if ov not in visible_vars:
+            if not self.is_var_visible_in_block(ov):
                 msg = (
                     "Var {} is not visible in block {} and thus cannot "
                     + "be a block output.\n{}"
                 )
                 raise ValueError(msg.format(ov.name, self.name, self))
 
-        for ov in self._outputs:
+        # For duplicate vars in self._outputs, only remove block once.
+        for ov in set(self._outputs):
             ov.consuming_blocks.remove(self)
 
         # Need to copy, or block's output would be completely tied to a var's
         # output and we cannot replace a block output with another var's
         # output.
         self._outputs = copy.copy(outputs)
-        for ov in outputs:
+        # For duplicate vars in outputs, only add consuming_blocks once.
+        for ov in set(outputs):
             ov.consuming_blocks.append(self)
 
     def __enter__(self):
@@ -260,141 +311,10 @@ class Block(object):
         return self
 
     def __exit__(self, type, value, traceback):
+        self._propagate_nonreplaceable_vars()
         global BLOCK_STACK
         BLOCK_STACK = BLOCK_STACK[:-1]
 
-    def _visible_vars_in_block(self, target_op=None, inclusive=True):
-        """
-        Returns
-        -------
-        - index (int) of target_op in self.operations if target_op not None,
-        undefined otherwise.
-
-        - visible_vars: set[Var]
-            Vars returned by ops in the block (self) visible (and equal to
-            if inclusive==True) target_op.  If target_op is not found or None,
-            include all vars output by self.operations. Examples:
-
-        Raises
-        ------
-        - ValueError if target_op not None and not found in self.operations.
-
-        #    main(%a: (1, 2, fp32),
-        #         %b: (1, 2, fp32),
-        #         %c: (1, 2, fp32)) {
-        #      block0() {
-        #        %const1: (1, fp32) = const(...)
-        #        %loop:0: (1, 2, fp32), %loop:1: (1, 2, fp32) = \
-        #        while_loop(loop_vars=(%a, %b))
-        #          loop_cond(%a.x, %b.x) {
-        #            %blah: (bool) = some_op(x=%a.x, y=%b.x)
-        #            %cond_var: (bool) = some_op2(x=%a.x, y=%blah)
-        #          } -> (%cond_var)
-        #          loop_body(%a.x, %b.x) {
-        #            %add_0: (1, 2, fp32) = add(x=%a.x, y=%b.x)
-        #          } -> (%add_0, %b.x)
-        #        %linear: (1, fp32) = linear(...)
-        #      } -> (%loop:0, %loop:1)
-        #    }
-        #
-
-        Let V0 and V1 be the set of internal_vars of block0 and loop_cond
-        block that supplies const vals (for const).
-
-        Ex1: self = block0, target_op = linear.
-        idx = 2
-        visible_vars = {%const1, %loop:0, %loop:1, %linear, V0}
-
-        Ex2: self = loop_cond, target_op = None.
-        idx = undefined
-        visible_vars = {%a.x, %b.x, %blah, %cond_var, V1}
-
-        Ex3: self = loop_cond, target_op = some_op.
-        idx = 0
-        visible_vars = {%a.x, %b.x, %blah, V1}
-
-        Ex4: self = loop_cond, target_op = linear.
-        raises ValueError (linear not found in loop_cond block)
-        """
-        visible_vars = set(self._internal_vars)
-        if isinstance(self, Function):
-            # Function inputs
-            visible_vars.update(tuple(self.inputs.values()))
-        else:
-            # Block inputs
-            visible_vars.update(self.inputs)
-
-        idx = -1
-        # find the location of target_op
-        for i, op in enumerate(self.operations):
-            if op == target_op:
-                if inclusive and op.outputs is not None:
-                    visible_vars.update(op.outputs)
-                return i, visible_vars
-            # When op is being constructed (e.g.,type_inference), op.outputs
-            # is None
-            if op.outputs is not None:
-                visible_vars.update(op.outputs)
-        if target_op is not None:
-            msg = "Op {} not found in {}: {}"
-            raise ValueError(msg.format(target_op.name, self.name, self))
-        return idx, visible_vars
-
-    def _visible_vars_from_enclosing_block(self):
-        """
-        Returns:
-
-        visible_vars: Vars from lexical scopes visible at the beginning of the
-        block, up to but not including outputs from before_op. Given program:
-
-        #    main(%a: (1, 2, fp32),
-        #         %b: (1, 2, fp32),
-        #         %c: (1, 2, fp32)) {
-        #      block0() {
-        #        %const1: (1, fp32) = const(...)
-        #        %loop:0: (1, 2, fp32), %loop:1: (1, 2, fp32) = \
-        #        while_loop(loop_vars=(%a, %b))
-        #          loop_cond(%a.x, %b.x) {
-        #            %blah: (bool) = some_op(x=%a.x, y=%b.x)
-        #            %cond_var: (bool) = some_op2(x=%a.x, y=%blah)
-        #          } -> (%cond_var)
-        #          loop_body(%a.x, %b.x) {
-        #            %add_0: (1, 2, fp32) = add(x=%a.x, y=%b.x)
-        #          } -> (%add_0, %b.x)
-        #        %const2: (1, fp32) = const(...)
-        #      } -> (%loop:0, %loop:1)
-        #    }
-
-        Let V0 be the set of internal_vars of block0 block that supplies const
-        vals (for const).
-
-        Ex1: self = block0
-            visible_vars = {%a, %b, %c} (function input)
-
-        Ex2: self = loop_cond.
-            visible_vars = {%a, %b, %c, %const1, V0} (Note that %const2 is not
-            part of the set)
-        """
-        visible_vars = set()
-
-        # function inputs are considered external to the block.
-        if isinstance(self, Function):
-            # block in function only has function_inputs as from enclosing
-            # block (Ex1 above).
-            visible_vars.update(self.function_inputs)
-            return visible_vars
-
-        if self.outer_op is not None:
-            enclosing_block = self.outer_op.enclosing_block
-            vars_at_start = enclosing_block._visible_vars_from_enclosing_block()
-            visible_vars.update(vars_at_start)
-            _, visible_vars_in_block = enclosing_block._visible_vars_in_block(
-                self.outer_op, inclusive=False
-            )
-            visible_vars.update(visible_vars_in_block)
-
-        return visible_vars
-
     def _insert_op_before(self, new_op, before_op=None):
         """
         A private API used by builder. Please use `builder.YOUR_OP(...,before_op)`.
@@ -428,26 +348,16 @@ class Block(object):
         is not visible before op0.
         """
         self.validate()
-        visible_vars = self._visible_vars_from_enclosing_block()
-        if before_op is not None:
-            idx, visible_vars_in_block = self._visible_vars_in_block(
-                before_op, inclusive=True
-            )
-            visible_vars.update(visible_vars_in_block)
-        else:
-            _, visible_vars_in_block = self._visible_vars_in_block()
-            visible_vars.update(visible_vars_in_block)
+
+        idx = len(self.operations) if before_op is None else self.find_op_id_in_block(before_op)
 
         # check inputs are visible
         for k, v in new_op.inputs.items():
             if not isinstance(v, (Var, tuple)):
                 continue
-            if isinstance(v, Var):
-                vs = [v]
-            else:
-                vs = v
+            vs = [v] if isinstance(v, Var) else v
             for s in vs:
-                if s not in visible_vars:
+                if not self.is_var_visible_in_block(s, upto_op_with_id=idx):
                     before_op_name = before_op.name if before_op is not None else "None"
                     msg = "Op '{}' input {}={} is not in scope of {} before {}"
                     raise ValueError(
@@ -507,19 +417,64 @@ class Block(object):
             self._block_inputs = tuple(self._block_inputs)
 
         # If old_var is block's output, replace as well.
-        if old_var in self._outputs:
-            idx = self._outputs.index(old_var)
-            self._outputs[idx] = new_var
-            new_var.consuming_blocks.append(self)
+        self.replace_block_output_var(old_var, new_var)
 
+        return num_ops_affected
+
+    def replace_block_output_var(
+            self,
+            old_var,
+            new_var,
+    ):
+        """
+        If old_var is in the list of block's outputs,
+        replace old_var with the new_var.
+        """
+        found_old_var_in_output = False
+        # There could be multiple matched `old_var` in output when the program has duplicate vars
+        # in the output.
+        for idx, output_var in enumerate(self._outputs):
+            if old_var == output_var:
+                found_old_var_in_output = True
+                self._outputs[idx] = new_var
+        if found_old_var_in_output:
+            new_var.consuming_blocks.append(self)
             # This block no longer uses `old_var` as its outputs
             old_var.consuming_blocks.remove(self)
-
-            # if rename_new_var_if_fn_output:
             # Ensure output name is consistent
             if isinstance(self, Function):
                 new_var.name = old_var.name
-        return num_ops_affected
+
+    def try_replace_uses_of_var_after_op(
+        self,
+        anchor_op,
+        old_var,
+        new_var,
+        no_check_var_types=False
+    ):
+        """
+        :param anchor_op: Operation
+        :param old_var: Var
+        :param new_var: Var
+        :param no_check_var_types: bool
+        :return: True if the old_var can be replaced by new_var. False otherwsie.
+
+        This helper function guards the replace_uses_of_var_after_op function,
+        by first checking if the old_var could be replaced by the new_var.
+
+        1. If old_var can be replaced by new_var, the replace_uses_of_var_after_op is called,
+        and returns True. 2. Return False if the replacement is not allow.
+        """
+        if not old_var.can_be_replaced_by_var(new_var):
+            return False
+
+        self.replace_uses_of_var_after_op(
+            anchor_op=anchor_op,
+            old_var=old_var,
+            new_var=new_var,
+            no_check_var_types=no_check_var_types,
+        )
+        return True
 
     def replace_uses_of_var_after_op(
         self,
@@ -529,6 +484,7 @@ class Block(object):
         no_check_var_visibility=False,
         end_op=None,
         no_check_var_types=False,
+        force_replace=False,
     ):
         """
         Replace all uses of `old_var` with `new_var` after `anchor_op`,
@@ -538,16 +494,17 @@ class Block(object):
         The op that produces the `old_var` will continue to produce it, its output
         won't be replaced by `new_var`.
 
-        If `anchor_op` is None, replace all input occurrences of `old_var` in the block.
-        If `end_op` is None, all occurrences of `old_var` are replaced in the block starting from the op just
-        after `anchor_op`
+        If `anchor_op` is None, replace all input occurrences of `old_var` in the block. If
+        `end_op` is None, all occurrences of `old_var` are replaced in the block starting from
+        the op just after `anchor_op`
 
         no_check_var_visibility: True to disable the check ensuring new_var is visible
         (visibility requirement depends on anchor_op).
 
-        no_check_var_types: An error will be raised if the type of new_var is not same as the old_var, unless
-        `no_check_var_types` is set to True. Normally type inference is re-invoked for all the child ops of `old_var`
-         after updating it to `new_var`. However, this is skipped if `no_check_var_types` is set to True.
+        no_check_var_types: An error will be raised if the type of new_var is not same as the
+        old_var, unless `no_check_var_types` is set to True. Normally type inference is
+        re-invoked for all the child ops of `old_var` after updating it to `new_var`. However,
+        this is skipped if `no_check_var_types` is set to True.
 
         old_var, new_var must meet the following conditions:
 
@@ -606,39 +563,36 @@ class Block(object):
                  %5 = op3(%2)               # will continue using %2
 
         """
+        if not force_replace and old_var.op is not None and new_var.op is not None:
+            if not old_var.can_be_replaced_by_var(new_var):
+                old_nonreplaceable_vars = old_var.nonreplaceable_vars_upstream
+                new_nonreplaceable_vars = new_var.nonreplaceable_vars_upstream
+                err_var = None
+                for _var in old_nonreplaceable_vars:
+                    if _var not in new_nonreplaceable_vars:
+                        err_var = _var
+                        break
+                msg = (
+                    "var {} cannot be replaced by {}. Since the nonreplaceable var {} might "
+                    "potentially "
+                    "be removed during the replacement of those vars."
+                ).format(old_var, new_var, err_var)
+                raise ValueError(msg)
+
+        start = self.find_op_id_in_block(anchor_op) + 1 if anchor_op is not None else 0
+        end_id = self.find_op_id_in_block(end_op) if end_op is not None else -1
+
         if not no_check_var_visibility:
             self.validate()
-        # Get visible vars from enclosing block
-        visible_vars = self._visible_vars_from_enclosing_block()
 
-        if anchor_op is not None:
-            # Get visible vars from the current block
-            idx, block_vars = self._visible_vars_in_block(anchor_op, inclusive=True)
-            visible_vars.update(block_vars)
-
-            # start from the next op, excluding `anchor_op`
-            start = idx + 1
-        else:
-            _, block_vars = self._visible_vars_in_block()
-            visible_vars.update(block_vars)
-
-            visible_vars.update(self._block_inputs)
-            visible_vars.update(self._internal_vars)
-            # Perform replacement from beginning
-            start = 0
-
-        if not no_check_var_visibility and new_var not in visible_vars:
-            msg = (
-                "new_var '{}' is not visible in block '{}' at or before "
-                + "anchor_op '{}'"
-            )
-            anchor_op_name = "None" if anchor_op is None else anchor_op.name
-            raise ValueError(msg.format(new_var.name, self.name, anchor_op_name))
-
-        if end_op is not None:
-            end_id, _ = self._visible_vars_in_block(end_op, inclusive=True)
-        else:
-            end_id = -1
+            idx = start if anchor_op is not None else len(self.operations)
+            if not self.is_var_visible_in_block(new_var, upto_op_with_id=idx):
+                msg = (
+                    "new_var '{}' is not visible in block '{}' at or before "
+                    + "anchor_op '{}'"
+                )
+                anchor_op_name = "None" if anchor_op is None else anchor_op.name
+                raise ValueError(msg.format(new_var.name, self.name, anchor_op_name))
 
         if end_id != -1 and end_id < start:
             msg = "end_op '{}' comes before the anchor_op '{}'"
@@ -652,17 +606,28 @@ class Block(object):
             no_check_var_types=no_check_var_types,
         )
 
-        logging.debug("Num ops affected in replacing var: {}".format(num_ops_affected))
+        logger.debug("Num ops affected in replacing var: {}".format(num_ops_affected))
 
     def remove_ops(self, existing_ops):
         """
-        Remove `existing_ops` (list[Operation]) that must be pre-existing in
-        the block. Error if any other op in the block uses output Vars of
-        `existing_ops`
+        Remove ops in `existing_ops`.
+
+        Args: existing_ops: List[Operation]. All ops in this list must be pre-existing in the
+        block. It allows duplicated ops, but duplicated ops will only be removed once.
+
+        Raises:
+            ValueError if any `op` in `existing_ops` meets any of following conditions:
+              - `op` is not found in the block
+              - any other op in the block uses output Vars of `op`
+              - the output var is block's output
         """
         self.validate()
-        idxs = [-1] * len(existing_ops)
+
+        # Dedup ops because each op can only be deleted once.
         existing_ops_set = set(existing_ops)
+        existing_ops = list(existing_ops_set)
+        # Find the idx of each to-be-removed op, and raise errors if any op couldn't be found.
+        idxs = [-1] * len(existing_ops)
         for i, op in enumerate(self.operations):
             if op in existing_ops_set:
                 idxs[existing_ops.index(op)] = i
@@ -749,6 +714,18 @@ class Block(object):
 
         return used_ops[::-1]
 
+    def _propagate_nonreplaceable_vars(self):
+        def propagate_nonreplaceable_vars_block(block):
+            for op in list(block.operations):
+                for b in op.blocks:
+                    propagate_nonreplaceable_vars_block(b)
+                if op.outputs is None:
+                    continue
+                for o in op.outputs:
+                    o._reset_nonreplaceable_vars_upstream()
+                    o._set_nonreplaceable_vars_upstream()
+        propagate_nonreplaceable_vars_block(self)
+
     def indented_str(self, indent=None):
         if indent is None:
             indent = ""
@@ -830,11 +807,14 @@ class Function(Block):
     """
     """
 
-    def __init__(self, inputs):
+    def __init__(self, inputs, opset_version=None):
         """
         inputs: str -> placeholder
+        opset_version: AvailableTarget enum. Describes the opset version of the function
         """
         self.placeholder_inputs = inputs
+        self.opset_version = opset_version
+
         # str -> Var
         self._input_dict = OrderedDict()
         for k, v in self.placeholder_inputs.items():
@@ -850,13 +830,26 @@ class Function(Block):
                 for s in shapes:
                     if is_symbolic(s):
                         k_used_symbols.add(s)
-        super(Function, self).__init__()
+        super().__init__()
 
     # Override Block's input
     @property
     def inputs(self):
         return self._input_dict
 
+    @property
+    def opset_version(self):
+        return self._opset_version
+
+    @opset_version.setter
+    def opset_version(self, version):
+        if not (
+            isinstance(version, _target) or
+            version is None
+        ):
+            raise ValueError("opset_version must be type of coremltools.AvailableTarget")
+        self._opset_version = version
+
     def __repr__(self):
         return self.__str__()
 
@@ -864,8 +857,9 @@ class Function(Block):
         return self.to_str("function")
 
     def to_str(self, func_name="function"):
+        func_name = func_name + "[{}]".format(_OPSET[self.opset_version])
         if len(self._input_dict) == 0:
-            s = func_name + "() {"
+            s = func_name + "()"
         else:
             inputs = [(in_name, ph) for in_name, ph in self._input_dict.items()]
             s = func_name + "(" + str(inputs[0][1])
