@@ -3,24 +3,21 @@
 # Use of this source code is governed by a BSD-3-clause license that can be
 # found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import numpy as np
 
 from coremltools.converters.mil.mil import types
-from coremltools.converters.mil.mil.input_type import (
-    DefaultInputs,
-    InputSpec,
-    TensorInputType,
-)
-from coremltools.converters.mil._deployment_compatibility import AvailableTarget as target
-from coremltools.converters.mil.mil.operation import Operation
+from coremltools.converters.mil.mil.input_type import (DefaultInputs,
+                                                       InputSpec,
+                                                       TensorInputType)
 from coremltools.converters.mil.mil.ops.defs._op_reqs import register_op
-from coremltools.converters.mil.mil.ops.defs.iOS15.image_resizing import (
-    crop_resize as _crop_resize_iOS15,
-    resample as _resample_iOS15,
-    upsample_bilinear as _upsample_bilinear_iOS15,
-)
+from coremltools.converters.mil.mil.ops.defs.iOS15.image_resizing import \
+    crop_resize as _crop_resize_iOS15
+from coremltools.converters.mil.mil.ops.defs.iOS15.image_resizing import \
+    resample as _resample_iOS15
+from coremltools.converters.mil.mil.ops.defs.iOS15.image_resizing import \
+    upsample_bilinear as _upsample_bilinear_iOS15
 from coremltools.converters.mil.mil.ops.defs.iOS16 import _IOS16_TARGET
 
+
 @register_op(opset_version=_IOS16_TARGET)
 class resample(_resample_iOS15):
     """
