@@ -3,20 +3,25 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-from coremltools.converters.mil import testing_reqs
-from coremltools.converters.mil.testing_reqs import *
-from coremltools.converters.mil.testing_utils import get_core_ml_prediction
-from coremltools._deps import _IS_MACOS
-from .testing_utils import UNK_SYM, run_compare_builder
+import itertools
 
-backends = testing_reqs.backends
+import numpy as np
+import pytest
+
+from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil import types
+from coremltools.converters.mil.testing_reqs import backends, compute_units
+from coremltools.converters.mil.testing_utils import get_core_ml_prediction
+from coremltools.models.utils import _macos_version
+
+from .testing_utils import UNK_SYM, run_compare_builder
 
 
 class TestRandomBernoulli:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
 
         x_val = np.array([0.0], dtype=np.float32)
         input_placeholders = {"x": mb.placeholder(shape=x_val.shape)}
@@ -43,14 +48,14 @@ class TestRandomBernoulli:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rank, prob, dynamic",
+        "compute_unit, backend, rank, prob, dynamic",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [rank for rank in range(1, 6)],
             [1.0, 0.0],
@@ -58,7 +63,7 @@ class TestRandomBernoulli:
         ),
     )
     def test_builder_to_backend_stress(
-        self, use_cpu_only, backend, rank, prob, dynamic
+        self, compute_unit, backend, rank, prob, dynamic
     ):
         shape = np.random.randint(low=1, high=4, size=rank).astype(np.int32)
         x_val = np.array([0.0], dtype=np.float32)
@@ -101,7 +106,7 @@ class TestRandomBernoulli:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
@@ -112,9 +117,9 @@ class TestRandomCategorical:
         return e_data / e_data.sum()
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         x_val = np.array([1], dtype=np.int32)
         input_placeholders = {"x": mb.placeholder(shape=x_val.shape)}
         input_values = {"x": x_val}
@@ -138,16 +143,21 @@ class TestRandomCategorical:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
-
+    @pytest.mark.skipif(_macos_version() < (12, 0), reason="Can only get predictions for ml program on macOS 12+")
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, n_sample, n_class",
-        itertools.product([True, False], backends, [50000], [2, 10, 20]),
+        "compute_unit, backend, n_sample, n_class",
+        itertools.product(
+            compute_units,
+            backends,
+            [50000],
+            [2, 10, 20]
+        ),
     )
-    def test_builder_to_backend_stress(self, use_cpu_only, backend, n_sample, n_class):
+    def test_builder_to_backend_stress(self, compute_unit, backend, n_sample, n_class):
         output_name = "random_categorical"
         logits = np.random.rand(2, n_class)
         probs = [self.softmax(logits[0]), self.softmax(logits[1])]
@@ -163,34 +173,33 @@ class TestRandomCategorical:
                 )
             ]
 
-        if _IS_MACOS:
-            prediction = get_core_ml_prediction(
-                build, input_placeholders, input_values, backend=backend
-            )
+        prediction = get_core_ml_prediction(
+            build, input_placeholders, input_values, backend=backend, compute_unit=compute_unit,
+        )
 
-            ref0 = np.random.multinomial(n_sample, probs[0])
-            ref1 = np.random.multinomial(n_sample, probs[1])
+        ref0 = np.random.multinomial(n_sample, probs[0])
+        ref1 = np.random.multinomial(n_sample, probs[1])
 
-            pred0 = prediction[output_name].reshape(2, n_sample)[0]
-            pred1 = prediction[output_name].reshape(2, n_sample)[1]
+        pred0 = prediction[output_name].reshape(2, n_sample)[0]
+        pred1 = prediction[output_name].reshape(2, n_sample)[1]
 
-            # convert to bincount and validate probabilities
-            pred0 = np.bincount(np.array(pred0).astype(np.int), minlength=n_class)
-            pred1 = np.bincount(np.array(pred1).astype(np.int), minlength=n_class)
+        # convert to bincount and validate probabilities
+        pred0 = np.bincount(np.array(pred0).astype(np.int), minlength=n_class)
+        pred1 = np.bincount(np.array(pred1).astype(np.int), minlength=n_class)
 
-            assert np.allclose(np.true_divide(pred0, n_sample), probs[0], atol=1e-2)
-            assert np.allclose(
-                np.true_divide(pred0, n_sample),
-                np.true_divide(ref0, n_sample),
-                atol=1e-2,
-            )
+        assert np.allclose(np.true_divide(pred0, n_sample), probs[0], atol=1e-2)
+        assert np.allclose(
+            np.true_divide(pred0, n_sample),
+            np.true_divide(ref0, n_sample),
+            atol=1e-2,
+        )
 
-            assert np.allclose(np.true_divide(pred1, n_sample), probs[1], atol=1e-2)
-            assert np.allclose(
-                np.true_divide(pred1, n_sample),
-                np.true_divide(ref1, n_sample),
-                atol=1e-2,
-            )
+        assert np.allclose(np.true_divide(pred1, n_sample), probs[1], atol=1e-2)
+        assert np.allclose(
+            np.true_divide(pred1, n_sample),
+            np.true_divide(ref1, n_sample),
+            atol=1e-2,
+        )
 
         # Test probs input
         input_placeholders = {"x": mb.placeholder(shape=(2, n_class))}
@@ -203,38 +212,37 @@ class TestRandomCategorical:
                 )
             ]
 
-        if _IS_MACOS:
-            prediction = get_core_ml_prediction(
-                build, input_placeholders, input_values, backend=backend
-            )
+        prediction = get_core_ml_prediction(
+            build, input_placeholders, input_values, backend=backend, compute_unit=compute_unit
+        )
 
-            pred0 = prediction[output_name].reshape(2, n_sample)[0]
-            pred1 = prediction[output_name].reshape(2, n_sample)[1]
+        pred0 = prediction[output_name].reshape(2, n_sample)[0]
+        pred1 = prediction[output_name].reshape(2, n_sample)[1]
 
-            # convert to bincount and validate probabilities
-            pred0 = np.bincount(np.array(pred0).astype(np.int), minlength=n_class)
-            pred1 = np.bincount(np.array(pred1).astype(np.int), minlength=n_class)
+        # convert to bincount and validate probabilities
+        pred0 = np.bincount(np.array(pred0).astype(np.int), minlength=n_class)
+        pred1 = np.bincount(np.array(pred1).astype(np.int), minlength=n_class)
 
-            assert np.allclose(np.true_divide(pred0, n_sample), probs[0], atol=1e-2)
-            assert np.allclose(
-                np.true_divide(pred0, n_sample),
-                np.true_divide(ref0, n_sample),
-                atol=1e-2,
-            )
+        assert np.allclose(np.true_divide(pred0, n_sample), probs[0], atol=1e-2)
+        assert np.allclose(
+            np.true_divide(pred0, n_sample),
+            np.true_divide(ref0, n_sample),
+            atol=1e-2,
+        )
 
-            assert np.allclose(np.true_divide(pred1, n_sample), probs[1], atol=1e-2)
-            assert np.allclose(
-                np.true_divide(pred1, n_sample),
-                np.true_divide(ref1, n_sample),
-                atol=1e-2,
-            )
+        assert np.allclose(np.true_divide(pred1, n_sample), probs[1], atol=1e-2)
+        assert np.allclose(
+            np.true_divide(pred1, n_sample),
+            np.true_divide(ref1, n_sample),
+            atol=1e-2,
+        )
 
 
 class TestRandomNormal:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         x_val = np.array([0.0], dtype=np.float32)
         input_placeholders = {"x": mb.placeholder(shape=x_val.shape)}
         input_values = {"x": x_val}
@@ -264,14 +272,14 @@ class TestRandomNormal:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rank, mean, dynamic",
+        "compute_unit, backend, rank, mean, dynamic",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [rank for rank in range(1, 6)],
             [1.0, 0.0],
@@ -279,7 +287,7 @@ class TestRandomNormal:
         ),
     )
     def test_builder_to_backend_stress(
-        self, use_cpu_only, backend, rank, mean, dynamic
+        self, compute_unit, backend, rank, mean, dynamic
     ):
         shape = np.random.randint(low=1, high=4, size=rank).astype(np.int32)
         x_val = np.array([0.0], dtype=np.float32)
@@ -327,16 +335,16 @@ class TestRandomNormal:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
 
 class TestRandomUniform:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         x_val = np.array([0.0], dtype=np.float32)
         input_placeholders = {"x": mb.placeholder(shape=x_val.shape)}
         input_values = {"x": x_val}
@@ -366,14 +374,14 @@ class TestRandomUniform:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rank, low, high, dynamic",
+        "compute_unit, backend, rank, low, high, dynamic",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [rank for rank in range(1, 6)],
             [0.0],
@@ -382,7 +390,7 @@ class TestRandomUniform:
         ),
     )
     def test_builder_to_backend_stress(
-        self, use_cpu_only, backend, rank, low, high, dynamic
+        self, compute_unit, backend, rank, low, high, dynamic
     ):
         shape = np.random.randint(low=1, high=4, size=rank).astype(np.int32)
         x_val = np.array([0.0], dtype=np.float32)
@@ -430,6 +438,6 @@ class TestRandomUniform:
             input_values,
             expected_output_types,
             expected_outputs=expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
         )
