@@ -8,24 +8,24 @@ import itertools
 import numpy as np
 import pytest
 
-from .testing_utils import run_compare_builder
-
 import coremltools as ct
-from coremltools._deps import _HAS_TF_1, MSG_TF1_NOT_FOUND
-from coremltools.converters.mil import testing_reqs
-from coremltools.converters.mil.mil import Builder as mb, types
-from coremltools.converters.mil.testing_reqs import backends
+from coremltools._deps import _HAS_TF_2, MSG_TF2_NOT_FOUND
+from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil import types
+from coremltools.converters.mil.testing_reqs import backends, compute_units
 from coremltools.converters.mil.testing_utils import ssa_fn
 
-if _HAS_TF_1:
+from .testing_utils import run_compare_builder
+
+if _HAS_TF_2:
     import tensorflow as tf
 
 
 class TestScatter:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         data = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([1, 0], dtype=np.int32)
         updates = np.array([[5, 6, 7], [8, 9, 10]], dtype=np.float32)
@@ -50,16 +50,15 @@ class TestScatter:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
-    @pytest.mark.skipif(not testing_reqs._HAS_TF_1, reason=MSG_TF1_NOT_FOUND)
+    @pytest.mark.skipif(not _HAS_TF_2, reason=MSG_TF2_NOT_FOUND)
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rankData_rankIndices, accumulate_mode",
+        "compute_unit, backend, rankData_rankIndices, accumulate_mode",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [
                 (1, 2),
@@ -79,7 +78,7 @@ class TestScatter:
         ),
     )
     def test_builder_to_backend_programmatic(
-        self, use_cpu_only, backend, rankData_rankIndices, accumulate_mode
+        self, compute_unit, backend, rankData_rankIndices, accumulate_mode
     ):
         data_rank, indices_rank = rankData_rankIndices
         data_shape = np.random.randint(low=2, high=5, size=data_rank)
@@ -97,24 +96,22 @@ class TestScatter:
                 data=data, indices=indices, updates=updates, mode=accumulate_mode
             )
 
-        with tf.Graph().as_default(), tf.Session() as sess:
-            tf_output = tf.Variable(data)
-            sess.run(tf.global_variables_initializer())
-            if accumulate_mode == "update":
-                sess.run(tf.scatter_update(tf_output, indices, updates))
-            if accumulate_mode == "add":
-                sess.run(tf.scatter_add(tf_output, indices, updates))
-            if accumulate_mode == "sub":
-                sess.run(tf.scatter_sub(tf_output, indices, updates))
-            if accumulate_mode == "mul":
-                sess.run(tf.scatter_mul(tf_output, indices, updates))
-            if accumulate_mode == "div":
-                sess.run(tf.scatter_div(tf_output, indices, updates))
-            if accumulate_mode == "max":
-                sess.run(tf.scatter_max(tf_output, indices, updates))
-            if accumulate_mode == "min":
-                sess.run(tf.scatter_min(tf_output, indices, updates))
-            expected_output = sess.run(tf_output)
+        tf_output = tf.Variable(data)
+        if accumulate_mode == "update":
+            tf.compat.v1.scatter_update(tf_output, indices, updates)
+        if accumulate_mode == "add":
+            tf.compat.v1.scatter_add(tf_output, indices, updates)
+        if accumulate_mode == "sub":
+            tf.compat.v1.scatter_sub(tf_output, indices, updates)
+        if accumulate_mode == "mul":
+            tf.compat.v1.scatter_mul(tf_output, indices, updates)
+        if accumulate_mode == "div":
+            tf.compat.v1.scatter_div(tf_output, indices, updates)
+        if accumulate_mode == "max":
+            tf.compat.v1.scatter_max(tf_output, indices, updates)
+        if accumulate_mode == "min":
+            tf.compat.v1.scatter_min(tf_output, indices, updates)
+        expected_output = tf_output.numpy()
 
         input_placeholders = {
             "data": mb.placeholder(shape=data.shape),
@@ -131,17 +128,16 @@ class TestScatter:
             input_values,
             expected_output_types,
             expected_output,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
 
 class TestScatterAlongAxis:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         data = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([[1, 0, 1], [1, 1, 0]], dtype=np.int32)
         updates = np.array([[5, 6, 7], [8, 9, 10]], dtype=np.float32)
@@ -168,8 +164,7 @@ class TestScatterAlongAxis:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
@@ -184,14 +179,14 @@ class TestScatterAlongAxis:
         np.testing.assert_allclose(np.array([[1, 6, 10], [8, 9, 7]], dtype=np.float32), v.val, atol=1e-04, rtol=1e-05)
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rank_axis",
+        "compute_unit, backend, rank_axis",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [(rank, axis) for rank in range(1, 5) for axis in range(-rank, rank)],
         ),
     )
-    def test_builder_to_backend_programmatic(self, use_cpu_only, backend, rank_axis):
+    def test_builder_to_backend_programmatic(self, compute_unit, backend, rank_axis):
         rank, axis = rank_axis
         data_shape = np.random.randint(low=2, high=8, size=rank)
         indices_shape = np.copy(data_shape)
@@ -228,17 +223,16 @@ class TestScatterAlongAxis:
             input_values,
             expected_output_types,
             np_output,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
 
 class TestScatterNd:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         data = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([[1, 0], [0, 2]], dtype=np.int32)
         updates = np.array([5, 10], dtype=np.float32)
@@ -263,16 +257,15 @@ class TestScatterNd:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
-    @pytest.mark.skipif(not testing_reqs._HAS_TF_1, reason=MSG_TF1_NOT_FOUND)
+    @pytest.mark.skipif(not _HAS_TF_2, reason=MSG_TF2_NOT_FOUND)
     @pytest.mark.parametrize(
-        "use_cpu_only, backend, rankData_rankIndices, accumulate_mode",
+        "compute_unit, backend, rankData_rankIndices, accumulate_mode",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [
                 (1, 2),
@@ -292,7 +285,7 @@ class TestScatterNd:
         ),
     )
     def test_builder_to_backend_programmatic(
-        self, use_cpu_only, backend, rankData_rankIndices, accumulate_mode
+        self, compute_unit, backend, rankData_rankIndices, accumulate_mode
     ):
         data_rank, indices_rank = rankData_rankIndices
         data_shape = np.random.randint(low=2, high=5, size=data_rank)
@@ -315,16 +308,14 @@ class TestScatterNd:
                 data=data, indices=indices, updates=updates, mode=accumulate_mode
             )
 
-        with tf.Graph().as_default(), tf.Session() as sess:
-            tf_output = tf.Variable(data)
-            sess.run(tf.global_variables_initializer())
-            if accumulate_mode == "update":
-                sess.run(tf.scatter_nd_update(tf_output, indices, updates))
-            if accumulate_mode == "add":
-                sess.run(tf.scatter_nd_add(tf_output, indices, updates))
-            if accumulate_mode == "sub":
-                sess.run(tf.scatter_nd_sub(tf_output, indices, updates))
-            expected_output = sess.run(tf_output)
+        tf_output = tf.Variable(data)
+        if accumulate_mode == "update":
+            tf.compat.v1.scatter_nd_update(tf_output, indices, updates)
+        if accumulate_mode == "add":
+            tf.compat.v1.scatter_nd_add(tf_output, indices, updates)
+        if accumulate_mode == "sub":
+            tf.compat.v1.scatter_nd_sub(tf_output, indices, updates)
+        expected_output = tf_output.numpy()
 
         input_placeholders = {
             "data": mb.placeholder(shape=data.shape),
@@ -341,17 +332,16 @@ class TestScatterNd:
             input_values,
             expected_output_types,
             expected_output,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
 
 class TestGather:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([1, 0], dtype=np.int32)
         input_placeholders = {
@@ -397,16 +387,15 @@ class TestGather:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke_iOS16(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke_iOS16(self, compute_unit, backend):
         if backend[0] == "neuralnetwork":
             pytest.skip("nn backend not supported")
         if ct.utils._macos_version() < (13, 0):
@@ -505,7 +494,7 @@ class TestGather:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
             minimum_deployment_target=ct.target.iOS16,
         )
@@ -530,9 +519,9 @@ class TestGather:
         )
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_embedding_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_embedding_builder_to_backend_smoke(self, compute_unit, backend):
         x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([1, 0], dtype=np.int32)
         input_placeholders = {
@@ -563,8 +552,7 @@ class TestGather:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
@@ -578,9 +566,9 @@ class TestGather:
 
 class TestGatherAlongAxis:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([[1, 0, 1], [1, 1, 0]], dtype=np.int32)
         input_placeholders = {
@@ -621,8 +609,7 @@ class TestGatherAlongAxis:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
@@ -634,15 +621,15 @@ class TestGatherAlongAxis:
         np.testing.assert_allclose(np.array([[4, 2, 6], [1, 2, 6]], dtype=np.float32), v.val, atol=1e-04, rtol=1e-05)
 
     @pytest.mark.parametrize(
-        "use_cpu_for_conversion, backend, rank_axis",
+        "compute_unit, backend, rank_axis",
         itertools.product(
-            [True, False],
+            compute_units,
             backends,
             [(rank, axis) for rank in range(1, 5) for axis in range(-rank, rank)],
         ),
     )
-    def test_builder_to_backend_programmatic(self, use_cpu_for_conversion, backend, rank_axis):
-        if backend[0] == "mlprogram" and not use_cpu_for_conversion:
+    def test_builder_to_backend_programmatic(self, compute_unit, backend, rank_axis):
+        if backend[0] == "mlprogram" and compute_unit != ct.ComputeUnit.CPU_ONLY:
             pytest.xfail("rdar://97398875 (TestGatherAlongAxis failing on mlprgram + GPU)")
         rank, axis = rank_axis
         x_shape = np.random.randint(low=2, high=8, size=rank)
@@ -673,17 +660,16 @@ class TestGatherAlongAxis:
             input_values,
             expected_output_types,
             expected_output,
-            use_cpu_only=use_cpu_for_conversion,
-            frontend_only=False,
+            compute_unit=compute_unit,
             backend=backend,
         )
 
 
 class TestGatherNd:
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke(self, compute_unit, backend):
         x = np.array([[1, 2, 3], [4, 5, 6]], dtype=np.float32)
         indices = np.array([[1, 0], [0, 2]], dtype=np.int32)
         input_placeholders = {
@@ -705,15 +691,15 @@ class TestGatherNd:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             frontend_only=False,
             backend=backend,
         )
 
     @pytest.mark.parametrize(
-        "use_cpu_only, backend", itertools.product([True, False], backends,)
+        "compute_unit, backend", itertools.product(compute_units, backends,)
     )
-    def test_builder_to_backend_smoke_iOS16(self, use_cpu_only, backend):
+    def test_builder_to_backend_smoke_iOS16(self, compute_unit, backend):
         if backend[0] == "neuralnetwork":
             pytest.skip("nn backend not supported")
 
@@ -758,7 +744,7 @@ class TestGatherNd:
             input_values,
             expected_output_types,
             expected_outputs,
-            use_cpu_only=use_cpu_only,
+            compute_unit=compute_unit,
             backend=backend,
             minimum_deployment_target=ct.target.iOS16,
         )
