@@ -3,13 +3,13 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import logging
-import os
-import pytest
-
+import coremltools as ct
+from coremltools import _logger as logger
+from coremltools.converters.mil.mil import Function, Program
 from coremltools.converters.mil.mil.types.symbolic import is_symbolic
-from coremltools.converters.mil.mil import Program, Function
-from coremltools.converters.mil.testing_utils import compare_backend, ct_convert
+from coremltools.converters.mil.testing_utils import (compare_backend,
+                                                      ct_convert)
+
 
 UNK_VARIADIC = "*s_unk"
 UNK_SYM = "s_unk"
@@ -21,14 +21,15 @@ def run_compare_builder(
     input_values,
     expected_output_types=None,
     expected_outputs=None,
-    use_cpu_only=False,
+    compute_unit=ct.ComputeUnit.CPU_ONLY,
     frontend_only=False,
     backend=("neuralnetwork", "fp32"),
     atol=1e-04,
     rtol=1e-05,
     inputs=None,
     also_compare_shapes=False,
-    use_cpu_for_conversion=False,
+    converter=ct.convert,
+    minimum_deployment_target=None,
 ):
     """
     Inputs:
@@ -46,6 +47,8 @@ def run_compare_builder(
         - expected_output_types: list[(shape, builtin_type)] or (shape,
           builtin_type).  None skips type inference validation.
 
+        - compute_unit: Enum[ct.ComputeUnit]. Compute unit for the coreml model
+
         - expected_outputs: list[np.array] or np.array. Required iff
           frontend_only == False
 
@@ -53,10 +56,12 @@ def run_compare_builder(
 
         - inputs: type of inputs (either None (defaults to tensor) or [ct.ImageType])
 
-        - use_cpu_for_conversion: bool
-            Argument which is passed as is to the unified converter API.
-            That is, "ct.convert(...., useCPUOnly=use_cpu_for_conversion)"
-            It forces the model to be loaded on the CPU context, post conversion.
+        - converter: function
+            Reference to convert function to be used.
+            Default: ct.convert
+
+        - minimum_deployment_target : coremltools.target enumeration (optional)
+            A member of the ``coremltools.target`` enum.
 
     Returns:
         The converted mlmodel
@@ -68,7 +73,7 @@ def run_compare_builder(
         expected_outputs = [expected_outputs]
 
     prog = Program()
-    with Function(input_placeholders) as ssa_func:
+    with Function(input_placeholders, opset_version=minimum_deployment_target) as ssa_func:
         output_vars = build(**ssa_func.inputs)
         if isinstance(output_vars, tuple):
             output_vars = list(output_vars)
@@ -98,7 +103,7 @@ def run_compare_builder(
             )
         if UNK_VARIADIC in s[:-1]:
             msg = "Skip type checking for UNK_VARIADIC. Output shape: {} vs expected shape: {}"
-            logging.debug(msg.format(out_var.shape, s[:-1]))
+            logger.debug(msg.format(out_var.shape, s[:-1]))
             continue
         expected_shape = s[:-1]
         msg = "Output {} shape: expect {}, got {}. Program:\n{}".format(
@@ -118,8 +123,14 @@ def run_compare_builder(
         if output_shape != expected_shape:
             raise ValueError(msg)
 
-    mlmodel = ct_convert(prog, source="milinternal", convert_to=backend, inputs=inputs,
-                         useCPUOnly=use_cpu_for_conversion)
+    mlmodel = ct_convert(prog,
+                         converter=converter,
+                         source="milinternal",
+                         convert_to=backend,
+                         inputs=inputs,
+                         compute_units=compute_unit,
+                         minimum_deployment_target=minimum_deployment_target
+    )
 
     if frontend_only:
         return mlmodel
@@ -139,11 +150,10 @@ def run_compare_builder(
         mlmodel=mlmodel,
         input_key_values=input_values,
         expected_outputs=expected_outputs,
-        use_cpu_only=use_cpu_only,
         atol=atol,
         rtol=rtol,
         also_compare_shapes=also_compare_shapes,
         dtype=backend[1]
     )
-    
+
     return mlmodel
