@@ -1,18 +1,17 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2021, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-
-from coremltools.converters.mil.mil.passes.pass_registry import register_pass
-from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
-from coremltools.converters.mil.mil import Builder as mb
 import numpy as np
 
-def _try_to_transform(conv_op, bn_op, block):
+from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
+from coremltools.converters.mil.mil.passes.helper import block_context_manager
+from coremltools.converters.mil.mil.passes.pass_registry import register_pass
 
+
+def _try_to_transform(conv_op, bn_op, block):
     # get parameters from batch_norm layer
     gamma = bn_op.gamma.val
     beta = bn_op.beta.val
@@ -29,7 +28,12 @@ def _try_to_transform(conv_op, bn_op, block):
 
     # get type of the conv layer
     is_deconv = conv_op.op_type == 'conv_transpose'
-    is_conv_1d  = len(conv_weight.shape) == 3
+    # The deconv weight transpose axes is determined by the dimension of convolution.
+    # Conv1d should be [1, 0, 2], Conv2d should be [1, 0, 2, 3], Conv3d should be [1, 0, 2, 3, 4]
+    if not 3 <= len(conv_weight.shape) <= 5:
+        raise AssertionError(f"Only supports Conv1/2/3d, which means weight's dimension should between 3 and 5, "
+                             f"but got weight with {len(conv_weight.shape)} dimensions. ")
+    deconv_weight_transpose_axes = [1, 0] + [axis for axis in range(2, len(conv_weight.shape))]
 
     # D_in denotes the spatial dimensions for conv kernel weight
     # for conv_transpose, conv_weight has shape [Cin, Cout / groups, *D_in]
@@ -60,7 +64,7 @@ def _try_to_transform(conv_op, bn_op, block):
     new_conv_bias = []
 
     if is_deconv:
-        conv_weight = np.transpose(conv_weight, [1, 0, 2] if is_conv_1d else [1, 0, 2, 3])
+        conv_weight = np.transpose(conv_weight, deconv_weight_transpose_axes)
         conv_weight = np.reshape(conv_weight, [Cout, Cin // groups] + list(conv_weight.shape[2:]))
 
     for i in range(Cout):
@@ -86,11 +90,13 @@ def _try_to_transform(conv_op, bn_op, block):
 
     if is_deconv:
         new_conv_weight = np.reshape(new_conv_weight, [Cout // groups, Cin] + list(new_conv_weight.shape[2:]))
-        new_conv_weight = np.transpose(new_conv_weight, [1, 0, 2] if is_conv_1d else [1, 0, 2, 3])
+        new_conv_weight = np.transpose(new_conv_weight, deconv_weight_transpose_axes)
 
     # make sure the updated weight and bias have the same shape as the original ones
-    assert new_conv_weight.shape == origin_weight_shape, "conv weight should have the same shape before and after the fuse_conv_batchnorm pass."
-    assert new_conv_bias.shape == origin_bias_shape, "conv bias should have the same shape before and after the fuse_conv_batchnorm pass."
+    if new_conv_weight.shape != origin_weight_shape:
+        raise AssertionError("conv weight should have the same shape before and after the fuse_conv_batchnorm pass. ")
+    if new_conv_bias.shape != origin_bias_shape:
+        raise AssertionError("conv bias should have the same shape before and after the fuse_conv_batchnorm pass. ")
 
     # create a new conv op with the new bias value, copying rest of the attributes
     out_name = bn_op.outputs[0].name
@@ -106,12 +112,49 @@ def _try_to_transform(conv_op, bn_op, block):
     else:
         x = mb.conv(**conv_kargs)
 
-    bn_op.enclosing_block.replace_uses_of_var_after_op(
-        anchor_op=bn_op, old_var=bn_op.outputs[0], new_var=x
-    )
-    # Remove all the ops at once
-    block.remove_ops([conv_op, bn_op])
-    return True
+    if bn_op.enclosing_block.try_replace_uses_of_var_after_op(
+            anchor_op=bn_op,
+            old_var=bn_op.outputs[0],
+            new_var=x,
+    ):
+        bn_op.enclosing_block.remove_ops([conv_op, bn_op])
+        return True
+    return False
+
+
+@block_context_manager
+def _fuse_conv_batchnorm_block(block):
+    def _match_pattern(op):
+        if op.op_type == "conv" or op.op_type == "conv_transpose":
+            # abort fusion if op output is also a block output
+            if op.outputs[0] in op.enclosing_block.outputs:
+                return None
+            # find batch_norm op
+            child_ops = op.outputs[0].child_ops
+            if len(child_ops) == 1:
+                bn_op_candidate = list(child_ops)[0]
+                if bn_op_candidate.op_type == "batch_norm":
+                    return bn_op_candidate
+        return None
+
+    fusion_occurred = False
+    for op in list(block.operations):
+        for b in op.blocks:
+            block_changed = True
+            while block_changed:
+                block_changed = _fuse_conv_batchnorm_block(b)
+        if len(op.blocks) > 0:
+            # This op can't be conv or conv_transpose
+            continue
+
+        bn_op = _match_pattern(op)
+        if bn_op is not None:
+            fusion_occurred = _try_to_transform(op, bn_op, block)
+            # has to break as the downstream iterator is affected.
+            if fusion_occurred:
+                return fusion_occurred
+    return fusion_occurred
+
 
 @register_pass(namespace="common")
 class fuse_conv_batchnorm(AbstractGraphPass):
@@ -129,50 +172,9 @@ class fuse_conv_batchnorm(AbstractGraphPass):
         ...
 
     """
-    def __init__(self):
-        self.ops_to_skip = set()
-
-    def set_ops_to_skip(self, prog):
-        pass
-
-    def _fuse_conv_batchnorm_block(self, block):
-
-        def _match_pattern(op):
-            if op.op_type == "conv" or op.op_type == "conv_transpose":
-                # abort fusion if op output is also a block output
-                if op.outputs[0] in op.enclosing_block.outputs:
-                    return None
-                # find batch_norm op
-                child_ops = op.outputs[0].child_ops
-                if len(child_ops) == 1:
-                    bn_op_candidate = list(child_ops)[0]
-                    if bn_op_candidate.op_type == "batch_norm":
-                        return bn_op_candidate
-            return None
-
-        fusion_occurred = False
-        for op in list(block.operations):
-            for b in op.blocks:
-                block_changed = True
-                while block_changed:
-                    block_changed = self._fuse_conv_batchnorm_block(b)
-            if len(op.blocks) > 0:
-                # This op can't be conv or conv_transpose
-                continue
-            if op in self.ops_to_skip:
-                continue
-            bn_op = _match_pattern(op)
-            if bn_op is not None:
-                with block:
-                    fusion_occurred = _try_to_transform(op, bn_op, block)
-                # has to break as the downstream iterator is affected.
-                if fusion_occurred:
-                    return fusion_occurred
-        return fusion_occurred
 
     def apply(self, prog):
-        self.set_ops_to_skip(prog)
         for f in prog.functions.values():
             block_changed = True
             while block_changed:
-                block_changed = self._fuse_conv_batchnorm_block(f)
+                block_changed = _fuse_conv_batchnorm_block(f)
