@@ -1,16 +1,16 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2021, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-
-from coremltools.converters.mil.mil.passes.pass_registry import register_pass
-from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
-from coremltools.converters.mil.mil import Builder as mb
 import numpy as np
 
+from coremltools.converters.mil.mil import Builder as mb
+from coremltools.converters.mil.mil.passes.graph_pass import AbstractGraphPass
+from coremltools.converters.mil.mil.passes.helper import block_context_manager
+from coremltools.converters.mil.mil.passes.pass_registry import register_pass
+
+
 def _try_to_transform(conv_op, scale_op, block):
 
     # get the scale
@@ -42,7 +42,7 @@ def _try_to_transform(conv_op, scale_op, block):
 
     # get type of the conv layer
     is_deconv = conv_op.op_type == 'conv_transpose'
-    is_conv_1d  = len(conv_weight.shape) == 3
+    is_conv_1d = len(conv_weight.shape) == 3
 
     # D_in denotes the spatial dimensions for conv kernel weight
     # for conv_transpose, conv_weight has shape [Cin, Cout / groups, *D_in]
@@ -69,9 +69,9 @@ def _try_to_transform(conv_op, scale_op, block):
 
     # transform the scale to 1./scale for the real_div case
     if scale_op.op_type == "real_div":
-        scale = 1./scale
+        scale = 1. / scale
 
-     # get the type of the conv weight
+    # get the type of the conv weight
     conv_weight_type = conv_weight.dtype
 
     # create bias for conv if not exist
@@ -125,16 +125,17 @@ def _try_to_transform(conv_op, scale_op, block):
     else:
         x = mb.conv(**conv_kargs)
 
-    scale_op.enclosing_block.replace_uses_of_var_after_op(
-        anchor_op=scale_op, old_var=scale_op.outputs[0], new_var=x
-    )
-    # Remove all the ops at once
-    block.remove_ops([conv_op, scale_op])
-    return True
-
-
+    if scale_op.enclosing_block.try_replace_uses_of_var_after_op(
+        anchor_op=scale_op,
+        old_var=scale_op.outputs[0],
+        new_var=x,
+    ):
+        scale_op.enclosing_block.remove_ops([conv_op, scale_op])
+        return True
+    return False
+    
+@block_context_manager
 def _fuse_conv_scale_block(block):
-
     def _match_pattern(op):
         if op.op_type == "conv" or op.op_type == "conv_transpose":
             # abort fusion if op output is also a block output
@@ -159,14 +160,15 @@ def _fuse_conv_scale_block(block):
             continue
 
         scale_op = _match_pattern(op)
+
         if scale_op is not None:
-            with block:
-                fusion_occurred = _try_to_transform(op, scale_op, block)
+            fusion_occurred = _try_to_transform(op, scale_op, block)
             # has to break as the downstream iterator is affected.
             if fusion_occurred:
                 return fusion_occurred
     return fusion_occurred
 
+
 @register_pass(namespace="common")
 class fuse_conv_scale(AbstractGraphPass):
     """
@@ -187,6 +189,7 @@ class fuse_conv_scale(AbstractGraphPass):
         ...
 
     """
+
     def apply(self, prog):
         for f in prog.functions.values():
             block_changed = True
