@@ -3,27 +3,88 @@
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
 
-import logging as _logging
 import numpy as _np
 import sympy as _sm
 
+from coremltools import _logger as logger
+from coremltools.converters.mil._deployment_compatibility import \
+    AvailableTarget as _target
+from coremltools.converters.mil.input_types import InputType
+from coremltools.converters.mil.mil.ops.helper import _get_version_of_op
+
 from . import types
 from .block import Function
+from .types.symbolic import k_num_internal_syms, k_used_symbols
 from .var import Var
-from .types.symbolic import k_used_symbols, k_num_internal_syms
-from coremltools.converters.mil.input_types import InputType
 
 
 class Program:
     def __init__(self):
         self.main_input_types = []
+        self.main_output_types = None
         self.functions = {}
         self.parameters = {}
+        self.skip_all_passes = False
+
+    def _get_max_opset_version_and_op(self):
+        max_opset_version = _target.iOS13
+        op_with_max_opset_version = None
+        def update_max_opset_version_block(block):
+            nonlocal max_opset_version
+            nonlocal op_with_max_opset_version
+            for op in list(block.operations):
+                for b in op.blocks:
+                    update_max_opset_version_block(b)
+                if not hasattr(op, "_op_variants") or not isinstance(op._op_variants, dict):
+                    continue
+                if op.opset_version > max_opset_version:
+                    max_opset_version = op.opset_version
+                    op_with_max_opset_version = op
+        for func in self.functions.values():
+            update_max_opset_version_block(func)
+        return max_opset_version, op_with_max_opset_version
+
+    def _check_ops_version_compatibility(self, max_opset_version):
+        def check_version_compatibility_block(block):
+            for op in list(block.operations):
+                for b in op.blocks:
+                    check_version_compatibility_block(b)
+                if not hasattr(op, "_op_variants") or not isinstance(op._op_variants, dict):
+                    continue
+                expected_op_cls = _get_version_of_op(op._op_variants, max_opset_version)
+                if type(op) is not expected_op_cls:
+                    msg = (
+                        "Op {} with an out of date version {!s} is detected. Please use @mb.program(input_specs=..., "
+                        "opset_version={!s})"
+                    ).format(op.op_type, op.opset_version, max_opset_version)
+                    raise ValueError(msg)
+        for func in self.functions.values():
+            check_version_compatibility_block(func)
+
+    def _check_or_set_functions_opset_version(self, max_opset_version):
+        funcs = list(self.functions.values())
+        for func in funcs:
+            if func.opset_version is None:
+                func.opset_version = max_opset_version
+            else:
+                if func.opset_version < max_opset_version:
+                    msg = "function should have at least opset_version {!s}. Got {!s}".format(max_opset_version, func.opset_version)
+                    raise ValueError(msg)
+        for func in funcs:
+            if func.opset_version != funcs[0].opset_version:
+                msg = "all functions must have the same opset_version. Got {!s} and {!s}.".format(func.opset_version, funcs[0].opset_version)
+                raise ValueError(msg)
+
+    def _check_program_opset_version(self):
+        max_opset_version, _ = self._get_max_opset_version_and_op()
+        self._check_ops_version_compatibility(max_opset_version)
+        self._check_or_set_functions_opset_version(max_opset_version)
 
     def add_function(self, name, ssa_func):
         if not isinstance(ssa_func, Function):
             raise ValueError("Only Function can be added to Program.")
         self.functions[name] = ssa_func
+        self._check_program_opset_version()
 
     def add_parameters(self, name, ssa_val):
         raise NotImplementedError()
@@ -35,6 +96,13 @@ class Program:
             raise ValueError("main inputs should be tuple of InputSpec")
         self.main_input_types = inputs
 
+    def set_main_output_types(self, outputs=None):
+        if outputs is not None:
+            if not (isinstance(outputs, list) and all([isinstance(out, InputType) for out in outputs])):
+                raise TypeError("main outputs should be a list of type ct.TensorType or ct.ImageType")
+        self.main_output_types = outputs
+
+
     def find_ops(self, prefix=None, op_type=None, exactly_one=False):
         """
         Return list of ops with name matching `prefix` if specified, and
@@ -94,7 +162,7 @@ class Placeholder:
             if not allow_rank0_input:
                 raise ValueError('Rank-0 (input {}) is unsupported'.format(name))
             else:
-                _logging.warning('Rank-0 (input {}) is unsupported in coreml. You might run into error while\
+                logger.warning('Rank-0 (input {}) is unsupported in coreml. You might run into error while\
                 running this model'.format(name))
 
         for i, d in enumerate(sym_shape):
@@ -151,7 +219,7 @@ def get_new_symbol(name=None):
         if s in k_used_symbols:
             new_name = name + k_num_internal_syms
             msg = 'Symbol name "{}" already occupied. Renaming to {}'
-            _logging.warning(msg.format(name, new_name))
+            logger.warning(msg.format(name, new_name))
             s = Symbol(new_name)
     else:
         s = Symbol("is" + str(k_num_internal_syms))
