@@ -1,35 +1,33 @@
-# -*- coding: utf-8 -*-
-
 #  Copyright (c) 2020, Apple Inc. All rights reserved.
 #
 #  Use of this source code is governed by a BSD-3-clause license that can be
 #  found in the LICENSE.txt file or at https://opensource.org/licenses/BSD-3-Clause
-
-from .type_bool import bool as types_bool
-from .type_double import (
-    is_float,
-    fp16 as types_fp16,
-    fp32 as types_fp32,
-    fp64 as types_fp64,
-)
-from .type_list import is_list
-from .type_int import (
-    is_int,
-    int8 as types_int8,
-    int16 as types_int16,
-    int32 as types_int32,
-    int64 as types_int64,
-    uint8 as types_uint8,
-    uint16 as types_uint16,
-    uint32 as types_uint32,
-    uint64 as types_uint64,
-)
-from .type_str import str as types_str
-from .type_unknown import unknown
+import numpy as _np
 import numpy as np
 import sympy as sm
-import six
+
+import coremltools.proto.MIL_pb2 as _mil_pm
+from coremltools.converters.mil.mil import types as _mil_types
+
 from .get_type_info import get_type_info
+from .type_bool import bool as types_bool
+from .type_bool import is_bool
+from .type_double import fp16 as types_fp16
+from .type_double import fp32 as types_fp32
+from .type_double import fp64 as types_fp64
+from .type_double import is_float
+from .type_int import int8 as types_int8
+from .type_int import int16 as types_int16
+from .type_int import int32 as types_int32
+from .type_int import int64 as types_int64
+from .type_int import is_int
+from .type_int import uint8 as types_uint8
+from .type_int import uint16 as types_uint16
+from .type_int import uint32 as types_uint32
+from .type_int import uint64 as types_uint64
+from .type_list import is_list
+from .type_str import str as types_str
+from .type_unknown import unknown
 
 _types_TO_NPTYPES = {
     types_bool: np.bool_,
@@ -49,20 +47,49 @@ _types_TO_NPTYPES = {
 
 _types_TO_STRINGS = {
     types_bool: "bool",
-    types_int8: "i8",
-    types_int16: "i16",
-    types_int32: "i32",
-    types_int64: "i64",
-    types_uint8: "u8",
-    types_uint16: "u16",
-    types_uint32: "u32",
-    types_uint64: "u64",
+    types_int8: "int8",
+    types_int16: "int16",
+    types_int32: "int32",
+    types_int64: "int64",
+    types_uint8: "uint8",
+    types_uint16: "uint16",
+    types_uint32: "uint32",
+    types_uint64: "uint64",
     types_fp16: "fp16",
     types_fp32: "fp32",
     types_fp64: "fp64",
-    types_str: "str",
+    types_str: "string",
 }
 
+builtin_to_proto_types = {
+    # bool:
+    _mil_types.bool: _mil_pm.BOOL,
+
+    # fp
+    _mil_types.fp16: _mil_pm.FLOAT16,
+    _mil_types.float: _mil_pm.FLOAT32,
+    _mil_types.double: _mil_pm.FLOAT64,
+
+    # int
+    _mil_types.uint8: _mil_pm.UINT8,
+    _mil_types.int8: _mil_pm.INT8,
+
+    _mil_types.uint16: _mil_pm.UINT16,
+    _mil_types.int16: _mil_pm.INT16,
+
+    _mil_types.uint32: _mil_pm.UINT32,
+    _mil_types.int32: _mil_pm.INT32,
+
+    _mil_types.uint64: _mil_pm.UINT64,
+    _mil_types.int64: _mil_pm.INT64,
+
+    # str
+    _mil_types.str: _mil_pm.STRING,
+}
+
+proto_to_builtin_types = {v: k for k, v in builtin_to_proto_types.items()}
+
+
 def np_dtype_to_py_type(np_dtype):
     # Can't use dict, as hash(np.int32) != hash(val.dtype)
     if np_dtype in [np.int32, np.int64]:
@@ -107,6 +134,16 @@ def promote_types(dtype1, dtype2):
 
     Returns:
         A builtin datatype or None.
+
+    Examples:
+        >>> promote_types(int32, int64)
+            builtin('int64')
+
+        >>> promote_types(fp16, fp32)
+            builtin('fp32')
+
+        >>> promote_types(fp16, int32)
+            builtin('fp16')
     """
     nptype1 = nptype_from_builtin(dtype1)
     nptype2 = nptype_from_builtin(dtype2)
@@ -124,6 +161,34 @@ def promote_types(dtype1, dtype2):
     return numpy_type_to_builtin_type(nppromoted)
 
 
+def promote_dtypes(dtypes):
+    """
+    Get the smallest promoted dtype, to which all scalar dtypes (provided through dtypes list argument) can be casted.
+    Args:
+        List [dtype (builtin)]
+    Returns:
+        A builtin datatype or None.
+
+    Examples:
+        >>> promote_dtypes([int32, int64, int16])
+            builtin('int64')
+
+        >>> promote_dtypes([fp16, fp32, fp64])
+            builtin('fp64')
+
+        >>> promote_dtypes([fp16, int32, int64])
+            builtin('fp16')
+
+    """
+
+    if not isinstance(dtypes, (list, tuple)) or len(dtypes) < 1:
+        raise ValueError("dtypes needs to be a list/tuple of atleast 1 element")
+
+    if len(dtypes) == 1:
+        return dtypes[0]
+
+    return promote_types(dtypes[0], promote_dtypes(dtypes[1:]))
+
 def is_primitive(btype):
     """
     Is the indicated builtin type a primitive?
@@ -133,9 +198,9 @@ def is_primitive(btype):
 
 def is_scalar(btype):
     """
-    Is the given builtin type a scalar integer, float, or boolean?
+    Is the given builtin type a scalar integer, float, boolean or string?
     """
-    return btype is types_bool or is_int(btype) or is_float(btype)
+    return is_bool(btype) or is_int(btype) or is_float(btype) or is_str(btype)
 
 
 def is_tensor(tensor_type):
@@ -184,7 +249,8 @@ def is_builtin(t):
 # Converts a numpy type to its types equivalent.
 # Supports both dtypes and numpy primitive types.
 def numpy_type_to_builtin_type(nptype):
-    if type(nptype) == np.dtype:
+    # If this is a data type object, use the corresponding scalar data type.
+    if np.issubclass_(type(nptype), np.dtype):
         nptype = nptype.type
 
     if np.issubclass_(nptype, np.bool) or np.issubclass_(nptype, np.bool_):
@@ -224,7 +290,7 @@ def numpy_type_to_builtin_type(nptype):
     elif np.issubclass_(nptype, np.float64) or np.issubclass_(nptype, np.double):
         return types_fp64
     elif (
-        np.issubclass_(nptype, six.string_types)
+        np.issubclass_(nptype, str)
         or np.issubclass_(nptype, np.string_)
         or np.issubclass_(nptype, np.str_)
     ):
@@ -243,9 +309,9 @@ def type_to_builtin_type(type):
     # Otherwise, try to infer from a few generic python types
     if np.issubclass_(type, bool):
         return types_bool
-    elif np.issubclass_(type, six.integer_types):
+    elif np.issubclass_(type, int):
         return types_int32
-    elif np.issubclass_(type, six.string_types):
+    elif np.issubclass_(type, str):
         return types_str
     elif np.issubclass_(type, float):
         return types_fp32
@@ -308,3 +374,24 @@ def is_subtype(type1, type2):
     if is_tensor(type1) and is_tensor(type2):
         return is_subtype_tensor(type1, type2)
     return type1 == type2
+
+
+def np_val_to_py_type(val):
+    """Convert numpy val to python primitive equivalent. Ex:
+
+    Given: val = np.array([True, False])
+    Returns: [True, False]
+
+    Given: val = np.array(32, dtype=np.int)
+    Returns 32
+    """
+    if not isinstance(val, (_np.ndarray, _np.generic)):
+        return val
+
+    if val.dtype in [_np.float16, _np.uint8, _np.int8, _np.uint32]:
+        return val.tobytes()
+    else:
+        # val is np.ndarray or np.generic
+        is_np_scalar = isinstance(val, _np.generic) or val.shape == ()
+        py_type = np_dtype_to_py_type(val.dtype)
+        return py_type(val) if is_np_scalar else tuple(py_type(v) for v in val.flatten())
\ No newline at end of file
